// MIT License
//
// Copyright (c) 2017-2019 MessageKit
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.

import Foundation
import MessageKit
import UIKit
// MARK: - TSChatMessage
internal class TSChatMessage: MessageType{
  // MARK: Lifecycle

    init(kind: MessageKind, user: TSChatUser, messageId: String, date: Date) {
        self.kind = kind
        self.user = user
        self.messageId = messageId
        sentDate = date
    }

    convenience init(custom: Any?, user: TSChatUser, messageId: String, date: Date) {
        self.init(kind: .custom(custom), user: user, messageId: messageId, date: date)
    }

    convenience init(text: String, user: TSChatUser, messageId: String, date: Date) {
        self.init(kind: .text(text), user: user, messageId: messageId, date: date)
    }

    convenience init(attributedText: NSAttributedString, user: TSChatUser, messageId: String, date: Date) {
        self.init(kind: .attributedText(attributedText), user: user, messageId: messageId, date: date)
    }

    convenience init(linkItem: LinkItem, user: TSChatUser, messageId: String, date: Date) {
        self.init(kind: .linkPreview(linkItem), user: user, messageId: messageId, date: date)
    }

    // MARK: Internal
    var messageId: String
    var sentDate: Date
    var kind: MessageKind
    var user: TSChatUser
    var sender: SenderType {
        user
    }
    var sendState: TSProgressState = .none
    var appendDict:[TSChatMessageAppendDictKey:Any] = [:]
    
    var messageType:TSChatMessageMsgType = .aiRobotResponse
    var markDownText:String = ""
}

// 定义一个枚举作为字典的键
enum TSChatMessageAppendDictKey: String ,Equatable{
    case topView
    case centerView
    case bottomView
    
    case centerBottomView
}

// 定义一个枚举作为消息体
enum TSChatMessageMsgType: String ,Equatable{
    case aiRobotWelcome     = "aiRobotWelcome"
    case userQuestion       = "userQuestion"
    case aiRobotResponse    = "aiRobotResponse"
}
